# -*- coding: utf-8 -*-
import unittest

from zwift import Client
from zwift.activity import Activity
from zwift.profile import Profile
from zwift.world import World


class TestZwift(unittest.TestCase):

    def setUp(self):
        username = 'Debbie'
        password = 'Harry'
        self.player_id = 123456
        self.client = Client(username, password)

    def test_activity(self):
        activity = self.client.get_activity(self.player_id)
        assert isinstance(activity, Activity)
        assert activity.player_id == self.player_id

    def test_profile(self):
        profile = self.client.get_profile(self.player_id)
        assert isinstance(profile, Profile)
        assert profile.player_id == self.player_id

    def test_profile_me(self):
        profile = self.client.get_profile()
        assert isinstance(profile, Profile)
        assert profile.player_id == 'me'

    def test_world(self):
        world = self.client.get_world()
        assert isinstance(world, World)
        assert world.world_id == 1  # default world id



# -*- coding: utf-8 -*-
from fitparse import FitFile, StandardUnitsDataProcessor

from .request import Request, download_file


class Activity:
    def __init__(self, player_id, get_access_token):
        self.player_id = player_id
        self.request = Request(get_access_token)

    def list(self, start=0, limit=20):
        return self.request.json(
            '/api/profiles/{}/activities/?start={}&limit={}'.format(self.player_id, start, limit))

    def get_data(self, activity_id):
        activity_data = self.get_activity(activity_id)
        fit_file_bucket = activity_data['fitFileBucket']
        fit_file_key = activity_data['fitFileKey']
        fit_file_url = 'https://{}.s3.amazonaws.com/{}'.format(
            fit_file_bucket, fit_file_key)
        raw_fit_data = download_file(fit_file_url)
        records = decode_fit_file(raw_fit_data)
        return process_fit_data(records)

    def get_activity(self, activity_id):
        return self.request.json(
            '/api/profiles/{}/activities/{}'.format(
                self.player_id, activity_id))


def decode_fit_file(raw_fit_data):
    fit_file = FitFile(
        raw_fit_data, data_processor=StandardUnitsDataProcessor())
    return [m for m in fit_file.get_messages() if m.name == 'record']


def process_fit_data(records):
    return [parse_fit_record(r) for r in records]


def parse_fit_record(record):
    return {
        'time': record.get_value('timestamp'),  # datetime in UTC
        'lat': record.get_value('position_lat'),  # WGS84
        'lng': record.get_value('position_long'),  # WGS84
        'altitude': record.get_value('enhanced_altitude'),  # m
        'distance': record.get_value('distance'),  # km
        'speed': record.get_value('enhanced_speed'),  # km/h
        'power': record.get_value('power'),  # watt
        'cadence': record.get_value('cadence'),  # rpm
        'heartrate': record.get_value('heart_rate'),  # bpm
    }



# -*- coding: utf-8 -*-
"""Top-level package for Zwift Mobile API client."""
try:
    from .client import Client
except ImportError:
    pass
else:
    __all__ = [Client]

__author__ = """Sander Smits"""
__email__ = 'jhmsmits@gmail.com'
__version__ = '0.2.0'



# -*- coding: utf-8 -*-
from .request import Request


class Profile:
    def __init__(self, player_id, get_access_token):
        self.player_id = player_id
        self.request = Request(get_access_token)

    @property
    def profile(self):
        return self.request.json('/api/profiles/{}'.format(self.player_id))

    def check_player_id(self):
        """Most /api/profiles endpoints require the real player id."""
        if self.player_id == 'me':
            profile = self.profile
            self.player_id = profile['id']

    @property
    def followers(self):
        self.check_player_id()
        return self.request.json(
            '/api/profiles/{}/followers'.format(self.player_id))

    @property
    def followees(self):
        self.check_player_id()
        return self.request.json(
            '/api/profiles/{}/followees'.format(self.player_id))

    def get_activities(self, start=0, limit=10):
        self.check_player_id()
        return self.request.json(
            '/api/profiles/{}/activities?start={}&limit={}'.format(
                self.player_id, start, limit))

    @property
    def latest_activity(self):
        activities = self.get_activities(0, 1)
        return activities[0] if len(activities) == 1 else None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: zwift_messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='zwift_messages.proto',
  package='',
  syntax='proto3',
  serialized_pb=_b('\n\x14zwift_messages.proto\"\xc6\x03\n\x0bPlayerState\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x11\n\tworldTime\x18\x02 \x01(\x03\x12\x10\n\x08\x64istance\x18\x03 \x01(\x05\x12\x10\n\x08roadTime\x18\x04 \x01(\x05\x12\x0c\n\x04laps\x18\x05 \x01(\x05\x12\r\n\x05speed\x18\x06 \x01(\x05\x12\x14\n\x0croadPosition\x18\x08 \x01(\x05\x12\x12\n\ncadenceUHz\x18\t \x01(\x05\x12\x11\n\theartrate\x18\x0b \x01(\x05\x12\r\n\x05power\x18\x0c \x01(\x05\x12\x0f\n\x07heading\x18\r \x01(\x03\x12\x0c\n\x04lean\x18\x0e \x01(\x05\x12\x10\n\x08\x63limbing\x18\x0f \x01(\x05\x12\x0c\n\x04time\x18\x10 \x01(\x05\x12\x0b\n\x03\x66\x31\x39\x18\x13 \x01(\x05\x12\x0b\n\x03\x66\x32\x30\x18\x14 \x01(\x05\x12\x10\n\x08progress\x18\x15 \x01(\x05\x12\x17\n\x0f\x63ustomisationId\x18\x16 \x01(\x03\x12\x14\n\x0cjustWatching\x18\x17 \x01(\x05\x12\x10\n\x08\x63\x61lories\x18\x18 \x01(\x05\x12\t\n\x01x\x18\x19 \x01(\x02\x12\x10\n\x08\x61ltitude\x18\x1a \x01(\x02\x12\t\n\x01y\x18\x1b \x01(\x02\x12\x17\n\x0fwatchingRiderId\x18\x1c \x01(\x05\x12\x0f\n\x07groupId\x18\x1d \x01(\x05\x12\r\n\x05sport\x18\x1f \x01(\x03\"\xd1\x01\n\x0e\x43lientToServer\x12\x11\n\tconnected\x18\x01 \x01(\x05\x12\x10\n\x08rider_id\x18\x02 \x01(\x05\x12\x12\n\nworld_time\x18\x03 \x01(\x03\x12\x1b\n\x05state\x18\x07 \x01(\x0b\x32\x0c.PlayerState\x12\r\n\x05seqno\x18\x04 \x01(\x05\x12\x0c\n\x04tag8\x18\x08 \x01(\x03\x12\x0c\n\x04tag9\x18\t \x01(\x03\x12\x13\n\x0blast_update\x18\n \x01(\x03\x12\r\n\x05tag11\x18\x0b \x01(\x03\x12\x1a\n\x12last_player_update\x18\x0c \x01(\x03\"\xe2\x01\n\rSegmentResult\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x10\n\x08rider_id\x18\x02 \x01(\x03\x12\x19\n\x11\x65vent_subgroup_id\x18\x06 \x01(\x03\x12\x12\n\nfirst_name\x18\x07 \x01(\t\x12\x11\n\tlast_name\x18\x08 \x01(\t\x12\x17\n\x0f\x66inish_time_str\x18\n \x01(\t\x12\x12\n\nelapsed_ms\x18\x0b \x01(\x03\x12\x12\n\npowermeter\x18\x0c \x01(\x05\x12\x0e\n\x06weight\x18\r \x01(\x05\x12\r\n\x05power\x18\x0f \x01(\x05\x12\x11\n\theartrate\x18\x13 \x01(\x05\"z\n\x0eSegmentResults\x12\x10\n\x08world_id\x18\x01 \x01(\x03\x12\x12\n\nsegment_id\x18\x02 \x01(\x03\x12\x19\n\x11\x65vent_subgroup_id\x18\x03 \x01(\x03\x12\'\n\x0fsegment_results\x18\x04 \x03(\x0b\x32\x0e.SegmentResult\"\x11\n\x0fUnknownMessage1\"\x10\n\x0eUnknownMessage\"\xe1\x01\n\x0eServerToClient\x12\x0c\n\x04tag1\x18\x01 \x01(\x05\x12\x10\n\x08rider_id\x18\x02 \x01(\x05\x12\x12\n\nworld_time\x18\x03 \x01(\x03\x12\r\n\x05seqno\x18\x04 \x01(\x05\x12#\n\rplayer_states\x18\x08 \x03(\x0b\x32\x0c.PlayerState\x12\'\n\x0eplayer_updates\x18\t \x03(\x0b\x32\x0f.UnknownMessage\x12\r\n\x05tag11\x18\x0b \x01(\x03\x12\r\n\x05tag17\x18\x11 \x01(\x03\x12\x10\n\x08num_msgs\x18\x12 \x01(\x05\x12\x0e\n\x06msgnum\x18\x13 \x01(\x05\"u\n\x0fWorldAttributes\x12\x10\n\x08world_id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04tag3\x18\x03 \x01(\x03\x12\x0c\n\x04tag5\x18\x04 \x01(\x03\x12\x12\n\nworld_time\x18\x06 \x01(\x03\x12\x12\n\nclock_time\x18\x07 \x01(\x03\"$\n\x0eWorldAttribute\x12\x12\n\nworld_time\x18\x02 \x01(\x03\"\xa9\x01\n\x15\x45ventSubgroupProtobuf\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05rules\x18\x08 \x01(\x05\x12\r\n\x05route\x18\x16 \x01(\x05\x12\x0c\n\x04laps\x18\x19 \x01(\x05\x12\x15\n\rstartLocation\x18\x1d \x01(\x05\x12\r\n\x05label\x18\x1e \x01(\x05\x12\x10\n\x08paceType\x18\x1f \x01(\x05\x12\x12\n\njerseyHash\x18$ \x01(\x05\"\xf1\x01\n\x0fRiderAttributes\x12\n\n\x02\x66\x32\x18\x02 \x01(\x05\x12\n\n\x02\x66\x33\x18\x03 \x01(\x05\x12;\n\x10\x61ttributeMessage\x18\x04 \x01(\x0b\x32!.RiderAttributes.AttributeMessage\x12\x0f\n\x07theirId\x18\n \x01(\x05\x12\x0b\n\x03\x66\x31\x33\x18\r \x01(\x05\x1ak\n\x10\x41ttributeMessage\x12\x0c\n\x04myId\x18\x01 \x01(\x05\x12\x0f\n\x07theirId\x18\x02 \x01(\x05\x12\x11\n\tfirstName\x18\x03 \x01(\t\x12\x10\n\x08lastName\x18\x04 \x01(\t\x12\x13\n\x0b\x63ountryCode\x18\x05 \x01(\x05\"&\n\x08Profiles\x12\x1a\n\x08profiles\x18\x01 \x03(\x0b\x32\x08.Profile\"\x8a\x03\n\x07Profile\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x11\n\tfirstName\x18\x04 \x01(\t\x12\x10\n\x08lastName\x18\x05 \x01(\t\x12\x0c\n\x04male\x18\x06 \x01(\x05\x12\x0e\n\x06weight\x18\t \x01(\x05\x12\x10\n\x08\x62odyType\x18\x0c \x01(\x05\x12\x13\n\x0b\x63ountryCode\x18\" \x01(\x05\x12\x15\n\rtotalDistance\x18# \x01(\x05\x12\x1c\n\x14totalDistanceClimbed\x18$ \x01(\x05\x12\x1a\n\x12totalTimeInMinutes\x18% \x01(\x05\x12\x16\n\x0etotalWattHours\x18) \x01(\x05\x12\x0e\n\x06height\x18* \x01(\x05\x12\x1d\n\x15totalExperiencePoints\x18. \x01(\x05\x12\x18\n\x10\x61\x63hievementLevel\x18\x31 \x01(\x05\x12\x13\n\x0bpowerSource\x18\x34 \x01(\x05\x12\x0b\n\x03\x61ge\x18\x37 \x01(\x05\x12\x1a\n\x12launchedGameClient\x18l \x01(\t\x12\x19\n\x11\x63urrentActivityId\x18m \x01(\x05\x62\x06proto3')
)




_PLAYERSTATE = _descriptor.Descriptor(
  name='PlayerState',
  full_name='PlayerState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='PlayerState.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='worldTime', full_name='PlayerState.worldTime', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='distance', full_name='PlayerState.distance', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='roadTime', full_name='PlayerState.roadTime', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='laps', full_name='PlayerState.laps', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='speed', full_name='PlayerState.speed', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='roadPosition', full_name='PlayerState.roadPosition', index=6,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cadenceUHz', full_name='PlayerState.cadenceUHz', index=7,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='heartrate', full_name='PlayerState.heartrate', index=8,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='power', full_name='PlayerState.power', index=9,
      number=12, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='heading', full_name='PlayerState.heading', index=10,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lean', full_name='PlayerState.lean', index=11,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='climbing', full_name='PlayerState.climbing', index=12,
      number=15, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time', full_name='PlayerState.time', index=13,
      number=16, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='f19', full_name='PlayerState.f19', index=14,
      number=19, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='f20', full_name='PlayerState.f20', index=15,
      number=20, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='progress', full_name='PlayerState.progress', index=16,
      number=21, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='customisationId', full_name='PlayerState.customisationId', index=17,
      number=22, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='justWatching', full_name='PlayerState.justWatching', index=18,
      number=23, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='calories', full_name='PlayerState.calories', index=19,
      number=24, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='x', full_name='PlayerState.x', index=20,
      number=25, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='altitude', full_name='PlayerState.altitude', index=21,
      number=26, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='PlayerState.y', index=22,
      number=27, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='watchingRiderId', full_name='PlayerState.watchingRiderId', index=23,
      number=28, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='groupId', full_name='PlayerState.groupId', index=24,
      number=29, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sport', full_name='PlayerState.sport', index=25,
      number=31, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25,
  serialized_end=479,
)


_CLIENTTOSERVER = _descriptor.Descriptor(
  name='ClientToServer',
  full_name='ClientToServer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='connected', full_name='ClientToServer.connected', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rider_id', full_name='ClientToServer.rider_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='world_time', full_name='ClientToServer.world_time', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='ClientToServer.state', index=3,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='seqno', full_name='ClientToServer.seqno', index=4,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag8', full_name='ClientToServer.tag8', index=5,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag9', full_name='ClientToServer.tag9', index=6,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ClientToServer.last_update', index=7,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag11', full_name='ClientToServer.tag11', index=8,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_player_update', full_name='ClientToServer.last_player_update', index=9,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=482,
  serialized_end=691,
)


_SEGMENTRESULT = _descriptor.Descriptor(
  name='SegmentResult',
  full_name='SegmentResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='SegmentResult.id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rider_id', full_name='SegmentResult.rider_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='event_subgroup_id', full_name='SegmentResult.event_subgroup_id', index=2,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_name', full_name='SegmentResult.first_name', index=3,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_name', full_name='SegmentResult.last_name', index=4,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='finish_time_str', full_name='SegmentResult.finish_time_str', index=5,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='elapsed_ms', full_name='SegmentResult.elapsed_ms', index=6,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='powermeter', full_name='SegmentResult.powermeter', index=7,
      number=12, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='weight', full_name='SegmentResult.weight', index=8,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='power', full_name='SegmentResult.power', index=9,
      number=15, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='heartrate', full_name='SegmentResult.heartrate', index=10,
      number=19, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=694,
  serialized_end=920,
)


_SEGMENTRESULTS = _descriptor.Descriptor(
  name='SegmentResults',
  full_name='SegmentResults',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='world_id', full_name='SegmentResults.world_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='segment_id', full_name='SegmentResults.segment_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='event_subgroup_id', full_name='SegmentResults.event_subgroup_id', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='segment_results', full_name='SegmentResults.segment_results', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=922,
  serialized_end=1044,
)


_UNKNOWNMESSAGE1 = _descriptor.Descriptor(
  name='UnknownMessage1',
  full_name='UnknownMessage1',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1046,
  serialized_end=1063,
)


_UNKNOWNMESSAGE = _descriptor.Descriptor(
  name='UnknownMessage',
  full_name='UnknownMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1065,
  serialized_end=1081,
)


_SERVERTOCLIENT = _descriptor.Descriptor(
  name='ServerToClient',
  full_name='ServerToClient',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag1', full_name='ServerToClient.tag1', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rider_id', full_name='ServerToClient.rider_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='world_time', full_name='ServerToClient.world_time', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='seqno', full_name='ServerToClient.seqno', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='player_states', full_name='ServerToClient.player_states', index=4,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='player_updates', full_name='ServerToClient.player_updates', index=5,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag11', full_name='ServerToClient.tag11', index=6,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag17', full_name='ServerToClient.tag17', index=7,
      number=17, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_msgs', full_name='ServerToClient.num_msgs', index=8,
      number=18, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='msgnum', full_name='ServerToClient.msgnum', index=9,
      number=19, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1084,
  serialized_end=1309,
)


_WORLDATTRIBUTES = _descriptor.Descriptor(
  name='WorldAttributes',
  full_name='WorldAttributes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='world_id', full_name='WorldAttributes.world_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='WorldAttributes.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag3', full_name='WorldAttributes.tag3', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag5', full_name='WorldAttributes.tag5', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='world_time', full_name='WorldAttributes.world_time', index=4,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='clock_time', full_name='WorldAttributes.clock_time', index=5,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1311,
  serialized_end=1428,
)


_WORLDATTRIBUTE = _descriptor.Descriptor(
  name='WorldAttribute',
  full_name='WorldAttribute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='world_time', full_name='WorldAttribute.world_time', index=0,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1430,
  serialized_end=1466,
)


_EVENTSUBGROUPPROTOBUF = _descriptor.Descriptor(
  name='EventSubgroupProtobuf',
  full_name='EventSubgroupProtobuf',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='EventSubgroupProtobuf.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='EventSubgroupProtobuf.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rules', full_name='EventSubgroupProtobuf.rules', index=2,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='route', full_name='EventSubgroupProtobuf.route', index=3,
      number=22, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='laps', full_name='EventSubgroupProtobuf.laps', index=4,
      number=25, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='startLocation', full_name='EventSubgroupProtobuf.startLocation', index=5,
      number=29, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='label', full_name='EventSubgroupProtobuf.label', index=6,
      number=30, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='paceType', full_name='EventSubgroupProtobuf.paceType', index=7,
      number=31, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jerseyHash', full_name='EventSubgroupProtobuf.jerseyHash', index=8,
      number=36, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1469,
  serialized_end=1638,
)


_RIDERATTRIBUTES_ATTRIBUTEMESSAGE = _descriptor.Descriptor(
  name='AttributeMessage',
  full_name='RiderAttributes.AttributeMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='myId', full_name='RiderAttributes.AttributeMessage.myId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='theirId', full_name='RiderAttributes.AttributeMessage.theirId', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='firstName', full_name='RiderAttributes.AttributeMessage.firstName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lastName', full_name='RiderAttributes.AttributeMessage.lastName', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='countryCode', full_name='RiderAttributes.AttributeMessage.countryCode', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1775,
  serialized_end=1882,
)

_RIDERATTRIBUTES = _descriptor.Descriptor(
  name='RiderAttributes',
  full_name='RiderAttributes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='f2', full_name='RiderAttributes.f2', index=0,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='f3', full_name='RiderAttributes.f3', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='attributeMessage', full_name='RiderAttributes.attributeMessage', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='theirId', full_name='RiderAttributes.theirId', index=3,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='f13', full_name='RiderAttributes.f13', index=4,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_RIDERATTRIBUTES_ATTRIBUTEMESSAGE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1641,
  serialized_end=1882,
)


_PROFILES = _descriptor.Descriptor(
  name='Profiles',
  full_name='Profiles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='profiles', full_name='Profiles.profiles', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1884,
  serialized_end=1922,
)


_PROFILE = _descriptor.Descriptor(
  name='Profile',
  full_name='Profile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Profile.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='firstName', full_name='Profile.firstName', index=1,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lastName', full_name='Profile.lastName', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='male', full_name='Profile.male', index=3,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='weight', full_name='Profile.weight', index=4,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bodyType', full_name='Profile.bodyType', index=5,
      number=12, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='countryCode', full_name='Profile.countryCode', index=6,
      number=34, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalDistance', full_name='Profile.totalDistance', index=7,
      number=35, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalDistanceClimbed', full_name='Profile.totalDistanceClimbed', index=8,
      number=36, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalTimeInMinutes', full_name='Profile.totalTimeInMinutes', index=9,
      number=37, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalWattHours', full_name='Profile.totalWattHours', index=10,
      number=41, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='height', full_name='Profile.height', index=11,
      number=42, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='totalExperiencePoints', full_name='Profile.totalExperiencePoints', index=12,
      number=46, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='achievementLevel', full_name='Profile.achievementLevel', index=13,
      number=49, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='powerSource', full_name='Profile.powerSource', index=14,
      number=52, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='age', full_name='Profile.age', index=15,
      number=55, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='launchedGameClient', full_name='Profile.launchedGameClient', index=16,
      number=108, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='currentActivityId', full_name='Profile.currentActivityId', index=17,
      number=109, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1925,
  serialized_end=2319,
)

_CLIENTTOSERVER.fields_by_name['state'].message_type = _PLAYERSTATE
_SEGMENTRESULTS.fields_by_name['segment_results'].message_type = _SEGMENTRESULT
_SERVERTOCLIENT.fields_by_name['player_states'].message_type = _PLAYERSTATE
_SERVERTOCLIENT.fields_by_name['player_updates'].message_type = _UNKNOWNMESSAGE
_RIDERATTRIBUTES_ATTRIBUTEMESSAGE.containing_type = _RIDERATTRIBUTES
_RIDERATTRIBUTES.fields_by_name['attributeMessage'].message_type = _RIDERATTRIBUTES_ATTRIBUTEMESSAGE
_PROFILES.fields_by_name['profiles'].message_type = _PROFILE
DESCRIPTOR.message_types_by_name['PlayerState'] = _PLAYERSTATE
DESCRIPTOR.message_types_by_name['ClientToServer'] = _CLIENTTOSERVER
DESCRIPTOR.message_types_by_name['SegmentResult'] = _SEGMENTRESULT
DESCRIPTOR.message_types_by_name['SegmentResults'] = _SEGMENTRESULTS
DESCRIPTOR.message_types_by_name['UnknownMessage1'] = _UNKNOWNMESSAGE1
DESCRIPTOR.message_types_by_name['UnknownMessage'] = _UNKNOWNMESSAGE
DESCRIPTOR.message_types_by_name['ServerToClient'] = _SERVERTOCLIENT
DESCRIPTOR.message_types_by_name['WorldAttributes'] = _WORLDATTRIBUTES
DESCRIPTOR.message_types_by_name['WorldAttribute'] = _WORLDATTRIBUTE
DESCRIPTOR.message_types_by_name['EventSubgroupProtobuf'] = _EVENTSUBGROUPPROTOBUF
DESCRIPTOR.message_types_by_name['RiderAttributes'] = _RIDERATTRIBUTES
DESCRIPTOR.message_types_by_name['Profiles'] = _PROFILES
DESCRIPTOR.message_types_by_name['Profile'] = _PROFILE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PlayerState = _reflection.GeneratedProtocolMessageType('PlayerState', (_message.Message,), dict(
  DESCRIPTOR = _PLAYERSTATE,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:PlayerState)
  ))
_sym_db.RegisterMessage(PlayerState)

ClientToServer = _reflection.GeneratedProtocolMessageType('ClientToServer', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTTOSERVER,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:ClientToServer)
  ))
_sym_db.RegisterMessage(ClientToServer)

SegmentResult = _reflection.GeneratedProtocolMessageType('SegmentResult', (_message.Message,), dict(
  DESCRIPTOR = _SEGMENTRESULT,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:SegmentResult)
  ))
_sym_db.RegisterMessage(SegmentResult)

SegmentResults = _reflection.GeneratedProtocolMessageType('SegmentResults', (_message.Message,), dict(
  DESCRIPTOR = _SEGMENTRESULTS,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:SegmentResults)
  ))
_sym_db.RegisterMessage(SegmentResults)

UnknownMessage1 = _reflection.GeneratedProtocolMessageType('UnknownMessage1', (_message.Message,), dict(
  DESCRIPTOR = _UNKNOWNMESSAGE1,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:UnknownMessage1)
  ))
_sym_db.RegisterMessage(UnknownMessage1)

UnknownMessage = _reflection.GeneratedProtocolMessageType('UnknownMessage', (_message.Message,), dict(
  DESCRIPTOR = _UNKNOWNMESSAGE,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:UnknownMessage)
  ))
_sym_db.RegisterMessage(UnknownMessage)

ServerToClient = _reflection.GeneratedProtocolMessageType('ServerToClient', (_message.Message,), dict(
  DESCRIPTOR = _SERVERTOCLIENT,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:ServerToClient)
  ))
_sym_db.RegisterMessage(ServerToClient)

WorldAttributes = _reflection.GeneratedProtocolMessageType('WorldAttributes', (_message.Message,), dict(
  DESCRIPTOR = _WORLDATTRIBUTES,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:WorldAttributes)
  ))
_sym_db.RegisterMessage(WorldAttributes)

WorldAttribute = _reflection.GeneratedProtocolMessageType('WorldAttribute', (_message.Message,), dict(
  DESCRIPTOR = _WORLDATTRIBUTE,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:WorldAttribute)
  ))
_sym_db.RegisterMessage(WorldAttribute)

EventSubgroupProtobuf = _reflection.GeneratedProtocolMessageType('EventSubgroupProtobuf', (_message.Message,), dict(
  DESCRIPTOR = _EVENTSUBGROUPPROTOBUF,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:EventSubgroupProtobuf)
  ))
_sym_db.RegisterMessage(EventSubgroupProtobuf)

RiderAttributes = _reflection.GeneratedProtocolMessageType('RiderAttributes', (_message.Message,), dict(

  AttributeMessage = _reflection.GeneratedProtocolMessageType('AttributeMessage', (_message.Message,), dict(
    DESCRIPTOR = _RIDERATTRIBUTES_ATTRIBUTEMESSAGE,
    __module__ = 'zwift_messages_pb2'
    # @@protoc_insertion_point(class_scope:RiderAttributes.AttributeMessage)
    ))
  ,
  DESCRIPTOR = _RIDERATTRIBUTES,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:RiderAttributes)
  ))
_sym_db.RegisterMessage(RiderAttributes)
_sym_db.RegisterMessage(RiderAttributes.AttributeMessage)

Profiles = _reflection.GeneratedProtocolMessageType('Profiles', (_message.Message,), dict(
  DESCRIPTOR = _PROFILES,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:Profiles)
  ))
_sym_db.RegisterMessage(Profiles)

Profile = _reflection.GeneratedProtocolMessageType('Profile', (_message.Message,), dict(
  DESCRIPTOR = _PROFILE,
  __module__ = 'zwift_messages_pb2'
  # @@protoc_insertion_point(class_scope:Profile)
  ))
_sym_db.RegisterMessage(Profile)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
import requests

from .error import RequestException


def download_file(url):
    resp = requests.get(url)
    if not resp.ok:
        raise RequestException("{} - {}".format(
            resp.status_code, resp.reason))
    return resp.content


class Request:
    """Class for handling requests."""
    BASE_URL = 'https://us-or-rly101.zwift.com'
    DEFAULT_HEADERS = {
        "User-Agent": "Zwift/115 CFNetwork/758.0.2 Darwin/15.0.0"}

    def __init__(self, get_access_token):
        self.get_access_token = get_access_token

    def get_headers(self, accept_type):
        headers = {
            "Accept": accept_type,
            "Authorization": "Bearer " + self.get_access_token()
        }
        headers.update(self.DEFAULT_HEADERS)
        return headers

    def json(self, url):
        headers = self.get_headers(accept_type='application/json')
        resp = requests.get(self.BASE_URL + url, headers=headers)
        if not resp.ok:
            raise RequestException("{} - {}".format(
                resp.status_code, resp.reason))
        return resp.json()

    def protobuf(self, url):
        headers = self.get_headers(accept_type='application/x-protobuf-lite')
        resp = requests.get(self.BASE_URL + url, headers=headers)
        if not resp.ok:
            raise RequestException("{} - {}".format(
                resp.status_code, resp.reason))
        return resp.content



# -*- coding: utf-8 -*-
from . import zwift_messages_pb2
from .request import Request

COURSE_TO_WORLD = {3: 1, 4: 2, 5: 3, 6: 1}


class COURSES:
    WATOPIA = 3
    RICHMOND = 4
    LONDON = 5


class World:
    def __init__(self, world_id, get_access_token):
        self.world_id = world_id
        self.request = Request(get_access_token)

    @property
    def players(self):
        return self.request.json('/relay/worlds/{}'.format(self.world_id))

    def player_status(self, player_id):
        buffer = self.request.protobuf(
            '/relay/worlds/{}/players/{}'.format(self.world_id, player_id))
        player_state = zwift_messages_pb2.PlayerState()
        player_state.ParseFromString(buffer)
        return PlayerStateWrapper(player_state)


class PlayerStateWrapper(object):
    """
    Wrap a PlayerState instance to make it more usable.

    Fields provided by wrapped player_state:
    id, worldTime, distance, roadTime, laps, speed, roadPosition, cadenceUHz,
    heartrate, power, heading, lean, climbing, time, f19, f20, progress,
    customisationId, justWatching, calories, x, altitude, y, watchingRiderId,
    groupId, sport

    """
    class TURN_SIGNALS:
        RIGHT = 'right'
        LEFT = 'left'
        STRAIGHT = 'straight'

    def __init__(self, player_state):
        self.player_state = player_state

    @property
    def ride_ons(self):
        return (self.player_state.f19 >> 24) & 0xfff

    @property
    def is_turning(self):
        return (self.player_state.f19 & 8) != 0

    @property
    def is_forward(self):
        return (self.player_state.f19 & 4) != 0

    @property
    def course(self):
        return (self.player_state.f19 & 0xff0000) >> 16

    @property
    def world(self):
        return COURSE_TO_WORLD[self.course]

    @property
    def road_id(self):
        return (self.player_state.f20 & 0xff00) >> 8

    @property
    def road_direction(self):
        return (self.player_state.f20 & 0xffff000000) >> 24

    @property
    def turn_signal(self):
        signal_code = self.player_state.f20 & 0x70
        if signal_code == 0x10:
            return self.TURN_SIGNALS.RIGHT
        elif signal_code == 0x20:
            return self.TURN_SIGNALS.LEFT
        elif signal_code == 0x40:
            return self.TURN_SIGNALS.STRAIGHT
        else:
            return None

    @property
    def power_up(self):
        return self.player_state.f20 & 0xf

    @property
    def has_feather_boost(self):
        return self.power_up == 0

    @property
    def has_draft_boost(self):
        return self.power_up == 1

    @property
    def has_aero_boost(self):
        return self.power_up == 5

    @property
    def cadence(self):
        return int((self.player_state.cadenceUHz * 60) / 1000000)

    def __getattr__(self, item):
        """
        First try to get the requested item from the player_state. When it's
        not found, try to get it directly from the wrapper.
        """
        try:
            return getattr(self.player_state, item)
        except AttributeError:
            return self.__getattribute__(item)



# -*- coding: utf-8 -*-
from __future__ import print_function

import time

import requests


class AuthToken:
    def __init__(self, username, password):
        self.username = username
        self.password = password

        # variables from the API response
        self.access_token = None
        self.expires_in = None
        self.id_token = None
        self.not_before_policy = None
        self.refresh_token = None
        self.refresh_expires_in = None
        self.session_state = None
        self.token_type = None
        # absolute expire times of the access and refresh tokens
        self.access_token_expiration = None
        self.refresh_token_expiration = None

    def fetch_token_data(self):
        if self.have_valid_refresh_token():
            data = {
                "refresh_token": self.refresh_token,
                "grant_type": "refresh_token",
            }
        else:
            data = {
                "username": self.username,
                "password": self.password,
                "grant_type": "password",
            }
        data['client_id'] = "Zwift_Mobile_Link"
        r = requests.post(
            'https://secure.zwift.com/auth/realms/zwift/tokens/access/codes',
            data=data)
        if not r.ok:
            # TODO: handle exceptions
            pass
        return r.json()

    def update_token_data(self):
        """Parse the access token response."""
        token_data = self.fetch_token_data()
        now = time.time()
        for key, value in token_data.items():
            key = key.replace('-', '_')
            setattr(self, key, value)
        self.access_token_expiration = now + self.expires_in - 5
        self.refresh_token_expiration = now + self.refresh_expires_in - 5

    def have_valid_access_token(self):
        if not self.access_token or time.time() > self.access_token_expiration:
            return False
        else:
            return True

    def have_valid_refresh_token(self):
        if (not self.refresh_token or
                time.time() > self.refresh_token_expiration):
            return False
        else:
            return True

    def get_access_token(self):
        if self.have_valid_access_token():
            return self.access_token
        else:
            self.update_token_data()
            return self.access_token



# -*- coding: utf-8 -*-
from .activity import Activity
from .auth import AuthToken
from .profile import Profile
from .world import World


class Client:
    def __init__(self, username, password):
        self.auth_token = AuthToken(username, password)

    def get_activity(self, player_id):
        return Activity(player_id, self.auth_token.get_access_token)

    def get_profile(self, player_id='me'):
        return Profile(player_id, self.auth_token.get_access_token)

    def get_world(self, world_id=1):
        return World(world_id, self.auth_token.get_access_token)
